use Test::More tests => 8;
use Test::MockObject;
use DE_EPAGES::AmazonPayments::API::Resource::CaptureNotification;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Test::Mock::EpagesObject;
use strict;

RunScript( Message => 'long', Sub => sub {
    # Needs RunOnStore for it triggers a hook intTest?
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            my $MockLineItem = DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::AmazonPayments::API::Object::LineItemPaymentAmazonPayments', {
                Alias => 'MockLineItem',
                TransID2 => 'S02-3029547-9718378',
                TransStatus => 'CaptureOpen',
                Parent => DE_EPAGES::Test::Mock::EpagesObject->new(
                    'DE_EPAGES::Order::API::Object::LineItemContainer', {
                        Alias => 'LineItemContainer',
                        Parent => DE_EPAGES::Test::Mock::EpagesObject->new(
                            'DE_EPAGES::Order::API::Object::CustomerOrder', {
                                Alias => '10101',
                                Site => DE_EPAGES::Test::Mock::EpagesObject->new(
                                 'DE_EPAGES::Shop::API::Object::Shop', {
                                     Address => undef
                                 })
                        })
                })
        });

        my $CaptureRef = DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::Object::API::Object::Object', {
                Alias  => 'CaptureRef_1',
                Parent =>  DE_EPAGES::Test::Mock::EpagesObject->new(
                    'DE_EPAGES::Object::API::Object::Object', {
                        Alias => 'References',
                        Parent => $MockLineItem,
                    })
        });

        $MockLineItem->parent->set({Payment => $MockLineItem });
        $MockLineItem->mock('instanceOf', sub { 1 });

        my $MockShop = Test::MockObject->new;
        $MockShop->mock('loadObjectByGuid', sub {
            return $CaptureRef;
        });

        $MockLineItem->mock('closeOrder', sub {
            pass('called closeOrder');
        });

        DE_EPAGES::AmazonPayments::API::Resource::CaptureNotification->post($MockShop, {
            'captureId' => $MockLineItem->get('TransID2'),
            'captureReferenceId' => '5C3D108F376A42F6AB0174B34E195F5C',
            'creationTimestamp' => '2014-07-31T16:41:00Z',
            'captureStatus' => {
                'state' => 'Pending'
            },
            'captureAmount' => {
                'currency' => 'EUR',
                'amount' => '99.99'
            }
        });

        is( $MockLineItem->get('TransStatus'), 'CapturePending', 'TransStatus: Pending' );
        ok( $MockLineItem->parent->parent->get('PendingOn'), 'PendingOn status set');

        DE_EPAGES::AmazonPayments::API::Resource::CaptureNotification->post($MockShop, {
            'captureId' => $MockLineItem->get('TransID2'),
            'captureReferenceId' => '5C3D108F376A42F6AB0174B34E195F5C',
            'creationTimestamp' => '2014-07-31T16:41:00Z',
            'captureStatus' => {
                'state' => 'Completed'
            },
            'captureAmount' => {
                'currency' => 'EUR',
                'amount' => '99.99'
            }
        });

        is( $MockLineItem->get('TransStatus'), 'CaptureCompleted', 'TransStatus: Completed' );
        ok(!$MockLineItem->parent->parent->get('PendingOn'), 'PendingOn status unset');
        ok( $MockLineItem->parent->parent->get('PaidOn'), 'PaidOn status set');

        DE_EPAGES::AmazonPayments::API::Resource::CaptureNotification->post($MockShop, {
            'captureId' => $MockLineItem->get('TransID2'),
            'captureReferenceId' => '5C3D108F376A42F6AB0174B34E195F5C',
            'creationTimestamp' => '2014-07-31T16:41:00Z',
            'captureStatus' => {
                'state' => 'Declined'
            },
            'captureAmount' => {
                'currency' => 'EUR',
                'amount' => '99.99'
            }
        });
        is( $MockLineItem->get('TransStatus'), 'CaptureDeclined', 'TransStatus: Declined' );
        ok( $MockLineItem->parent->parent->get('PendingOn'), 'PendingOn status set');

    });
});
