use Test::More tests => 1;
use Test::MockObject;
use DE_EPAGES::AmazonPayments::API::Constants qw ( AUTHORIZATION_OPEN ORDER_CANCELED );
use DE_EPAGES::AmazonPayments::API::Resource::OrderNotification;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Test::Mock::EpagesObject;
use strict;

RunScript( Message => 'long', Sub => sub {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

        my $MockContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            Parent => DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', {}),
        });

        my $MockLineItem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemPayment', {
            Alias => 'MockLineItem',
            TransID => 'S02-3029547-9718378',
            TransStatus => AUTHORIZATION_OPEN,
            Parent => $MockContainer,
            Site => Test::MockObject->new,
        });

        $MockLineItem->mock('instanceOf', sub {
            my $self = shift;
            my ($ClassName) = @_;
            return $ClassName eq 'LineItemPaymentAmazonPayments';
        });

        local *DE_EPAGES::AmazonPayments::API::Resource::OrderNotification::LoadClassByAlias = sub {
            my $Mock = Test::MockObject->new;
            $Mock->mock('getObjects', sub { return [$MockLineItem]; });
            return $Mock;
        };

        DE_EPAGES::AmazonPayments::API::Resource::OrderNotification->post($MockLineItem->getSite(), {
            'id' => $MockLineItem->get('TransID'),
            'status' => {
                'state' => 'Canceled'
            },
        });

        is( $MockLineItem->get('TransStatus'), ORDER_CANCELED, 'TransStatus' );
    });
});
