#========================================================================================
# package      DE_EPAGES::EasyMarketing::t::unit::Snippet_Remarketing_ConfirmationPage
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::EasyMarketing::API::Snippet::Remarketing::ConfirmationPage
#========================================================================================
package DE_EPAGES::EasyMarketing::t::unit::Snippet_Remarketing_ConfirmationPage;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::EasyMarketing::API::Snippet::Remarketing::ConfirmationPage;
use DE_EPAGES::EasyMarketing::API::Snippet::Replacer;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Test::API::TestTools qw (TestExistsMethod);
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;
use strict;

sub test {
    my $self = shift;
    $self->test_ShopWithoutRemarketingCode();

    $self->test_ExpectedError();
    $self->test_NotExpectedError();

    $self->test_SnippetConfirmationPage_WrongObject();
    $self->test_SnippetConfirmationPage_Positive();

    done_testing();
}

## no critic (ProhibitLongLines)

#---------------------------------------------------------------------------
#  Shop without Remarketing Snippet
#---------------------------------------------------------------------------
sub test_ShopWithoutRemarketingCode {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createCustomerOrder();
    my $Shop = $Order->get('Site');
    $Shop->{'Attributes'} = {
            'EasyMarketingRemarketingCode' => undef,
    };

    my $ConfirmationPage = $self->_createRemarketingConfirmationPage();
    my $ExpectedSnippet = '';
    is($ConfirmationPage->getSnippet($Order), $ExpectedSnippet, "$SubTestName - Test if it return empty string when EasyMarketing attributes are not set for the shop");
}

#---------------------------------------------------------------------------
#  Test Expected Errors
#---------------------------------------------------------------------------
sub test_ExpectedError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createCustomerOrder();

    my $Replacer = $self->_createReplacerWithError('UnexpectedAmountOfMatchesInReplace');
    my $ConfirmationPage = $self->_createRemarketingConfirmationPage($Replacer);

    is($ConfirmationPage->getSnippet($Order), '', "$SubTestName - Test if it return empty string snippet for the confirmation page");

    my $ExpectedErrorCode = 'UnexpectedAmountOfMatchesInReplace';
    is_deeply($ConfirmationPage->{'_LogError'}, $ExpectedErrorCode, "$SubTestName - Test if it Log the expected Error");
}
#---------------------------------------------------------------------------
sub test_NotExpectedError {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createCustomerOrder();

    my $Replacer = $self->_createReplacerWithError('NotExpectedError');
    my $ConfirmationPage = $self->_createRemarketingConfirmationPage($Replacer);

    throws_ok( sub { $ConfirmationPage->getSnippet($Order) },
                   qr/NotExpectedError/,
                   "$SubTestName - Test if it throws an error with a not Order Object"
    );
}


#---------------------------------------------------------------------------
#  Order Confirmation Page
#---------------------------------------------------------------------------
sub test_SnippetConfirmationPage_WrongObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $NotOrderObject = 'NotOrderObject';

    my $Remarketing = $self->_createRemarketingConfirmationPage();

    throws_ok( sub { $Remarketing->getSnippet($NotOrderObject) },
                   qr/PARAMETER_NO_OBJECT: CustomerOrder/,
                   "$SubTestName - Test if it throws an error with a not Order Object"
    );
}
#---------------------------------------------------------------------------
sub test_SnippetConfirmationPage_Positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Order = $self->_createCustomerOrder();

    my $Replacer = $self->_createReplacer('snippetConfirmationPage');
    my $ConfirmationPage = $self->_createRemarketingConfirmationPage($Replacer);

    my $ExpectedSnippet = $self->_getExampleSnippet('ConfirmationPage');
    is($ConfirmationPage->getSnippet($Order), $ExpectedSnippet, "$SubTestName - Test if it return the snippet for the confirmation page");
    is_deeply($Replacer->{'processRemarketingSnippet_params'}, $self->_expectedReplacerConfirmationPage(), "$SubTestName - Test if the replacer receive the right params");
}
#---------------------------------------------------------------------------
#  Helpers - Mocked Injection points in the class
#---------------------------------------------------------------------------
sub _createReplacerWithError {
    my $self = shift;
    my ($ErrorName) = @_;
    my $MockedReplacer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::EasyMarketing::API::Snippet::Replacer', {});

    TestExistsMethod('DE_EPAGES::EasyMarketing::API::Snippet::Replacer','processRemarketingSnippet');
    $MockedReplacer->mock('processRemarketingSnippet',sub {
        Error($ErrorName);
    });
    return $MockedReplacer;
}
#---------------------------------------------------------------------------
sub _createReplacer {
    my $self = shift;
    my ($ReturnSnippet) = @_;
    my $MockedReplacer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::EasyMarketing::API::Snippet::Replacer', {});

    TestExistsMethod('DE_EPAGES::EasyMarketing::API::Snippet::Replacer','processRemarketingSnippet');
    $MockedReplacer->mock('processRemarketingSnippet',sub {
        shift;
        my ($RemarketingCode, $ProductIdList, $PageType, $BasketValue) = @_;
            $MockedReplacer->{'processRemarketingSnippet_params'} = {
               'RemarketingCode' => $RemarketingCode,
               'BasketValue' => $BasketValue,
               'PageType' => $PageType,
               'ProductId' => $ProductIdList
            };
        return $ReturnSnippet;
    });
    return $MockedReplacer;
}
#----------------------------------------------------------------------------------------
# Helpers - Mocked Remarketing different page types creation
#----------------------------------------------------------------------------------------
sub _createRemarketingConfirmationPage {
    my $self = shift;
    my ($Replacer) = @_;
    my $ConfirmationPage = DE_EPAGES::EasyMarketing::API::Snippet::Remarketing::ConfirmationPage->new();
    my $MockedRemarketing = Test::MockObject::Extends->new($ConfirmationPage);

    TestExistsMethod('DE_EPAGES::EasyMarketing::API::Snippet::Remarketing','_getReplacer');
    $MockedRemarketing->mock('_getReplacer',sub {
        return $Replacer;
    });
    TestExistsMethod('DE_EPAGES::EasyMarketing::API::Snippet::Remarketing','_LogError');
    $MockedRemarketing->mock('_LogError',sub {
        shift;
        my ($Error) = @_;
        $MockedRemarketing->{'_LogError'} = $Error->code();
        return;
    });
    return $MockedRemarketing;
}
#---------------------------------------------------------------------------
#  Helpers - Create ePages Objects
#---------------------------------------------------------------------------
sub _createCustomerOrder {
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'CustomerOrder',
        'Site' => $self->_createShop(),
        'LineItemContainer' => $self->_createLineItemContainer(),
    };
    my $MockedOrder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes);
    return $MockedOrder;
}
#---------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $LineItemProduct1 = $self->_createLineItemProduct($self->_createProduct('BasketProduct1'),1);
    my $ProductQuantity = 2;
    my $LineItemProduct2 = $self->_createLineItemProduct($self->_createProduct('BasketProduct2'),$ProductQuantity);
    my $NotLineItemProduct = $self->_createNotLineItemProduct();

    my $hAttributes = {
        'Alias' => 'My_LineItemContainer_Alias',
        'TotalBeforeTax' => '81.00',
        'CurrencyID' => 'EUR',
        'Children' => [$LineItemProduct1, $LineItemProduct2, $NotLineItemProduct]
    };

    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);

    return $MockedLineItemContainer;
}
#---------------------------------------------------------------------------
sub _createProduct{
    my $self = shift;
    my ($ProductAlias) = @_;
    my $hAttributes = {
        'Alias' => $ProductAlias,
        'instanceOf' => 'Product',
        'Site' => $self->_createShop(),
    };
    my $MockedProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', $hAttributes);

    use DE_EPAGES::Product::API::Object::Product;
    TestExistsMethod('DE_EPAGES::Product::API::Object::Product','price');
    $MockedProduct->mock('price',sub {
        return '31.00';
    });
    return $MockedProduct;
}
#---------------------------------------------------------------------------
sub _createNotLineItemProduct {
    my $self = shift;

    my $Product = $self->_createProduct('ProductOfNotLineItemProduct');
    my $hAttributes = {
        'Alias' => 'NotLineItemProduct',
        'instanceOf' => 'NotLineItemProduct',
        'Product' => $Product,
    };

    my $MockedLineItemProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemShipping', $hAttributes);

    return $MockedLineItemProduct;
}
#---------------------------------------------------------------------------
sub _createLineItemProduct {
    my $self = shift;
    my ($Product, $Quantity) = @_;

    my $ProductAlias = $Product->alias();
    my $hAttributes = {
        'Alias' => "LineItemProduct$ProductAlias",
        'instanceOf' => 'LineItemProduct',
        'Quantity' => $Quantity,
        'Product' => $Product,
    };

    my $MockedLineItemProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes);

    return $MockedLineItemProduct;
}
#---------------------------------------------------------------------------
sub _createShop{
    my $self = shift;
    my $hAttributes = {
        'EasyMarketingRemarketingCode' => 'RemarketingCode',
    };
    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $MockedShop;
}

#----------------------------------------------------------------------------------------
# Helpers - Expected results
#----------------------------------------------------------------------------------------
sub _expectedReplacerConfirmationPage {
    my $self = shift;
    my $hExpectedReplacedHash = {
       'BasketValue' => '81.00',
       'PageType' => 'purchase',
       'RemarketingCode' => 'RemarketingCode',
       'ProductId' => "['BasketProduct1', 'BasketProduct2']" ## no critic (ProhibitInterpolationOfLiterals)
    };
    return $hExpectedReplacedHash;
}
#---------------------------------------------------------------------------
sub _getExampleSnippet {
    my $self = shift;
    my ($SnippetType) = @_;

    my $ExpectedSnippet = '';
    $ExpectedSnippet .= "\n<!-- begin Remarketing$SnippetType -->\n";
    $ExpectedSnippet .= "snippet$SnippetType";
    $ExpectedSnippet .= "\n<!-- end Remarketing$SnippetType -->\n";

    return $ExpectedSnippet;
}
## use critic (ProhibitLongLines)

__PACKAGE__->runScript();

1;