#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::BillsafeArticleList
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for API::CGI::BillsafeArticleList
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::BillsafeArticleList;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::MockObject::Extends;
use Test::More;

use strict;



sub Main {

    Test_BillsafeArticleList();
    Test_BillsafeArticleListWithLineItemTax();

    done_testing();

}



sub Test_BillsafeArticleList {
    my $SubTestName = (caller(0))[3];

    my $BillsafeArticleList = DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList->new();
    $BillsafeArticleList = Test::MockObject::Extends->new($BillsafeArticleList);

    my $LineItemData = [
    {
        'Quantity' => '2',
        'LineItemPrice' => '3.14',
        'NameOrAlias' => 'TestProduct',
        'Class' => 'LineItemProduct',
        'TaxRate' => '0.19',
    },
    {
        'Quantity' => undef,
        'LineItemPrice' => '1.23',
        'NameOrAlias' => 'TestShipping',
        'Class' => 'LineItemShipping',
        'TaxRate' => '0.19',
    },
    ];


    $BillsafeArticleList->mock('_getDatabaseBasketOrOrder', sub {
        return _createDatabaseBasketOrOrder($LineItemData);
    });

    my $Order = _createOrder();
    my $ArticleList = $BillsafeArticleList->extractArticleList($Order);

    is($ArticleList, 'TestProduct@@2@@157@@goods@@19.00@@TestShipping@@1@@123@@shipment@@19.00', "$SubTestName - test if article list is correct"); ## no critic (RequireInterpolationOfMetachars, ProhibitLongLines)
}



sub Test_BillsafeArticleListWithLineItemTax {
    my $SubTestName = (caller(0))[3];

    my $BillsafeArticleList = DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList->new();
    $BillsafeArticleList = Test::MockObject::Extends->new($BillsafeArticleList);

    my $LineItemData = [
    {
        'Quantity' => '2',
        'LineItemPrice' => '3.14',
        'NameOrAlias' => 'TestProduct',
        'Class' => 'LineItemProduct',
        'TaxRate' => '0.19',
    },
    {
        'Quantity' => undef,
        'LineItemPrice' => '1.23',
        'NameOrAlias' => 'TestShipping',
        'Class' => 'LineItemShipping',
        'TaxRate' => '0.19',
    },
    {
        'Quantity' => undef,
        'LineItemPrice' => '1.35',
        'NameOrAlias' => 'Standard',
        'Class' => 'LineItemTax',
        'TaxRate' => '0.19',
    },
    ];


    $BillsafeArticleList->mock('_getDatabaseBasketOrOrder', sub {
        return _createDatabaseBasketOrOrder($LineItemData);
    });


    $BillsafeArticleList->mock('_GetInfoLanguage', sub {
        return {};
    });

    my $Order = _createOrder();
    my $ArticleList = $BillsafeArticleList->extractArticleList($Order);

    is($ArticleList, 'TestProduct@@2@@157@@goods@@19.00@@TestShipping@@1@@123@@shipment@@19.00@@{Tax} (19.00 %)@@1@@135@@goods@@19.00', "$SubTestName - test if article list is correct"); ## no critic (RequireInterpolationOfMetachars, ProhibitLongLines)
}



sub _createDatabaseBasketOrOrder {
    my ($LineItemData) = @_;

    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    my $MockedDatabaseBasketOrOrder = Test::MockObject::Extends->new($DatabaseBasketOrOrder);

    $MockedDatabaseBasketOrOrder->mock('extractFromBasketOrOrder', sub{
        return;
    });
    $MockedDatabaseBasketOrOrder->mock('getMinorUnitDigitCount', sub{
        return '2';
    });
    $MockedDatabaseBasketOrOrder->mock('getLanguageID', sub{
        return 'LANGID';
    });
    $MockedDatabaseBasketOrOrder->mock('getLineItemContainer', sub{
        return _createLineItemContainer($LineItemData);
    });

    return $MockedDatabaseBasketOrOrder;
}



# Order can be empty because it is only used over DatabaseBasketOrOrder which is mocked directly
sub _createOrder {
    return {};
}



sub _createLineItemContainer {
    my ($LineItemData) = @_;

    my $hLineItemContainer = {'LanguageID' => '2'};
    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hLineItemContainer);

    my $aChildObjects = [];
    for(my $i=0; $i<scalar(@{$LineItemData}); $i++) {
        push(@$aChildObjects, _createLineItem($LineItemData->[$i]));
    }

    $MockedLineItemContainer->mock('children', sub {
        return $aChildObjects;
    });

    return $MockedLineItemContainer;
}



sub _createLineItem {
    my ($hLineItemAttributes) = @_;

    my $MockedLineItem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItem', $hLineItemAttributes);

    $MockedLineItem->mock('instanceOf', sub {
        shift;
        my ($ClassName) = @_;
        my $IsObjectOfClass = $ClassName eq $MockedLineItem->get('Class');
        return $IsObjectOfClass;
    });

    return $MockedLineItem;
}



RunScript( Sub => \&Main );