#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::Request_HashBuilder;
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for HashBuilder
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Request_HashBuilder;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder;
use DE_EPAGES::Order::API::Object::Basket;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::MockObject::Extends;



sub Main {
    test_RequestData();
    done_testing();
}



sub test_RequestData {

    my $SubTestName = (caller(0))[3];

    my $HashBuilder = _GetHashBuilder_WithoutExternalBindings();
    my $Basket = _GetMockedBasket();

    my $hasRequestData = $HashBuilder->generateRequestData($Basket);
    ok($hasRequestData,  "$SubTestName - does the HashBuilder contain a hash?");

    my $hRequestData = $HashBuilder->getRequestData();
    my $hExpectedRequestData = _GetExpectedRequestData();
    is_deeply($hRequestData, $hExpectedRequestData, "$SubTestName - compare the request data for SOAP request to Ipayment");

}



sub _GetHashBuilder_WithoutExternalBindings {

    my $MockedDatabaseBasketOrOrder = _GetMockedDatabaseBasketOrOrder();
    my $HashBuilder = DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder->new($MockedDatabaseBasketOrOrder);
    my $HashBuilderMocked = Test::MockObject::Extends->new($HashBuilder);

    # mock _getMinorUnitDigitCount, so that we do not have to use the database for the unit test.
    $HashBuilderMocked->mock('_getMinorUnitDigitCount', sub {
        return 2; # EUR has 2 minor unit digits"
    });

    return $HashBuilderMocked;
}



sub _GetMockedDatabaseBasketOrOrder {

    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    my $MockedDatabaseBasketOrOrder = Test::MockObject::Extends->new($DatabaseBasketOrOrder);

    # mock _getDatabaseObjects, so that we do not have to use the database for the unit test.
    $MockedDatabaseBasketOrOrder->mock('_getDatabaseObjects', sub {
        return 1; ## no critic (ProhibitMagicNumbers)
    });

    $MockedDatabaseBasketOrOrder->mock('getAccountID', sub {
        return 99999; ## no critic (ProhibitMagicNumbers)
    });

    $MockedDatabaseBasketOrOrder->mock('getUserID', sub {
        return 99999; ## no critic (ProhibitMagicNumbers)
    });

    $MockedDatabaseBasketOrOrder->mock('getUserPassword', sub {
        return 0; ## no critic (ProhibitMagicNumbers)
    });

    $MockedDatabaseBasketOrOrder->mock('getAdminActionPassword', sub {
        return '5cfgRT34xsdedtFLdfHxj7tfwx24fe';
    });

    $MockedDatabaseBasketOrOrder->mock('getCurrencyID', sub {
        return 'EUR';
    });

    $MockedDatabaseBasketOrOrder->mock('getGrandTotal', sub {
        return 1.23; ## no critic (ProhibitMagicNumbers)
    });

    $MockedDatabaseBasketOrOrder->mock('getGUID', sub {
        return '53AC1BFE-5362-E914-FC55-AC1532072208';
    });

    $MockedDatabaseBasketOrOrder->mock('getTransactionType', sub {
        return 'auth';
    });
    $MockedDatabaseBasketOrOrder->mock('getPaymentType', sub {
        return 'elv';
    });
    $MockedDatabaseBasketOrOrder->mock('getRedirectUrl', sub {
        return '';
    });
    $MockedDatabaseBasketOrOrder->mock('getSilentErrorUrl', sub {
        return '';
    });
    $MockedDatabaseBasketOrOrder->mock('getHiddenTriggerUrl', sub {
        return '';
    });
    $MockedDatabaseBasketOrOrder->mock('_LoadRootObject', sub {
        my $hAttributes = {
            'Ipayment_PartnerId' => 'testPartnerId',
            'EpagesVersion' => '6.6.6',
        };
        my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hAttributes);
        return $MockedSystem;
    });
}



sub _GetMockedBasket {
    my $MockedBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', {});
    return $MockedBasket;
}



sub _GetExpectedRequestData {
    return {
        'accountData' =>
        {
            'accountId' => '99999',
            'trxuserId' => '99999',
            'trxpassword' => '0',
            'adminactionpassword' => '5cfgRT34xsdedtFLdfHxj7tfwx24fe',
        },
        'transactionData' =>
        {
            'trxAmount' => '123',
            'trxCurrency' => 'EUR',
            'invoiceText' => '',
            'trxUserComment' => '',
            'shopperId' => '53AC1BFE-5362-E914-FC55-AC1532072208',
        },
        'transactionType' => 'auth',
        'paymentType' => 'elv',
        'options' =>
        {
            'fromIp' => '',
            'advancedStrictIdCheck' => '1',
            'checkFraudattack' => '',
            'checkDoubleTrx' => '1',
            'errorLang' => '',
            'trxLongsafe' => '',
            'otherOptions' =>
            {
                'option' =>
                {
                    'key' => '',
                    'value' => ''
                }
            },
            'browserData' =>
            {
                'browserUserAgent' => '',
                'browserAcceptHeaders' => '',
            },
            'clientData' =>
            {
                'clientName' => 'ePages',
                'clientVersion' => '6.6.6',
                'commissionPartnerCode' => 'testPartnerId',
            }
        },
        'processorUrls' =>
        {
            'redirectUrl' => '',
            'silentErrorUrl' => '',
            'hiddenTriggerUrl' => ''
        }
    };
}



RunScript( Sub => \&Main );