#========================================================================================
# package      DE_EPAGES::Shippo::t::GrantShippoAuthorization_si
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for GrantShippoAuthorization
#========================================================================================
package DE_EPAGES::Shippo::t::GrantShippoAuthorization_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use Readonly;
use URI;
use URI::QueryParam;

use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Shippo::API::Connector;
use DE_EPAGES::Shippo::API::User qw ( CreateShippoUser GetShippoAuthToken );

use Test::MockObject::Extends;
use Test::More;

use strict;



# test plan
sub test {
    my $self = shift;

    $self->test_GrantShippoAuthorization();

    done_testing();
}

sub test_GrantShippoAuthorization {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopPath = $self->shopPath();
    my $Shop = LoadObjectByPath($ShopPath);

    my $Connector = $self->_createConnector($Shop);
    my $Password = 'password';
    my $User = CreateShippoUser($Shop, $Password);
    my $AuthToken = GetShippoAuthToken($User->pathString, $Password, $Connector->GetShippoSoapURL($Shop));
    my $ShippingFulfillmentType = $self->_getShippingFulfillmentType($Shop);

    is($ShippingFulfillmentType->get('AuthorizationURL'), 'https://goshippo.com/epages/auth/connect/',
       "$SubTestName - We are getting the URL of Shippo where the credentials have to be sent");
    like($User->pathString, '/ShippoConnector/' ,"$SubTestName - Check that a user called ShippoConnector is created");
    like($Connector->GetShippoSoapURL($Shop), '/soap$/' ,"$SubTestName - Check Shipo Soap URL sent"); ## no critic qw(RequireInterpolationOfMetachars)
    like($AuthToken, '/^[A-Z0-9]+$/i', "$SubTestName - Check that Authtoken contains data"); ## no critic qw(RequireInterpolationOfMetachars)
}

sub shopAlias {
    return 'TestShopShippo';
}

sub importXMLPath {
    return 'ShopXMLs/ShopWithShippo.xml';
}

#========================================================================================
sub _getShippingFulfillmentType {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippoShippingProviderMethod = $self->_getShippoShippingProviderMethod($Shop);
    return $ShippoShippingProviderMethod->get('ShippingFulfillmentType');
}

#========================================================================================
sub _createConnector {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippoShippingProviderMethod = $self->_getShippoShippingProviderMethod($Shop);
    return DE_EPAGES::Shippo::API::Connector->new($ShippoShippingProviderMethod);
}

#========================================================================================
sub _getShippoShippingProviderMethod {
    my $self = shift;
    my ($Shop) = @_;

    my $ProviderMethodsFolder = $Shop->folder('ShippingProviderMethods');
    my $ShippoProviderMethod;

    $ShippoProviderMethod = $ProviderMethodsFolder->child('Shippo') if ($ProviderMethodsFolder->existsChild('Shippo'));

    return $ShippoProviderMethod;
}

__PACKAGE__->runScript();

1;
